@@ -1,9 +1,10 @@
 """Data structures responsible for keeping track of mutations to a :class:`Value` or :class:`Collection`"""
 
-from tinychain import ref
-from tinychain.reflect import is_ref
-from tinychain.state import State
-from tinychain.util import get_ref, uri, URI
+from .reflect import is_ref
+from .scalar import ref
+from .scalar.value import Nil
+from .state import State
+from .util import uri, URI
 
 
 class Chain(State):
@@ -26,6 +27,17 @@ class Chain(State):
         else:
             raise ValueError(f"Chain subject must be a State, not {form}")
 
+    def __init__(self, form):
+        # in this case, the type of form is significant
+        # so we can't call State.__init__ because it will discard the type of the form
+
+        if isinstance(form, URI):
+            self.__uri__ = form
+        elif is_ref(form) and hasattr(form, "__uri__"):
+            self.__uri__ = uri(form)
+
+        self.__form__ = form
+
     def set(self, value):
         """Update the value of this `Chain`."""
 
@@ -33,23 +45,19 @@ class Chain(State):
 
     # TODO: delete these overrides and make MethodSubject compatible with Chain
     def _get(self, name, key=None, rtype=State):
-        from .value import Nil
         op_ref = ref.Get(uri(self).append(name), key)
         rtype = Nil if rtype is None else rtype
-        return rtype(op_ref)
+        return rtype(form=op_ref)
 
     def _put(self, name, key=None, value=None):
-        from .value import Nil
         return Nil(ref.Put(uri(self).append(name), key, value))
 
     def _post(self, name, params, rtype=State):
-        from .value import Nil
         op_ref = ref.Post(uri(self).append(name), params)
         rtype = Nil if rtype is None else rtype
-        return rtype(op_ref)
+        return rtype(form=op_ref)
 
     def _delete(self, name, key=None):
-        from .value import Nil
         return Nil(ref.Delete(uri(self).append(name), key))
 
 
