@@ -1,12 +1,13 @@
 """A `BTree` with a schema of named, :class:`Value`-typed :class:`Column` s."""
 
-from tinychain.ref import Ref
-from tinychain.state import Map, State, Stream, Tuple
-from tinychain.util import form_of, to_json, uri, URI
-from tinychain.value import UInt
+from ..generic import Map, Tuple
+from ..scalar.bound import Range
+from ..scalar.number import UInt
+from ..scalar.ref import Ref
+from ..state import State, Stream
+from ..util import form_of, to_json, uri, URI
 
-from .collection import Collection
-from .bound import Range
+from .base import Collection
 
 
 class Schema(object):
@@ -29,7 +30,7 @@ class BTree(Collection):
         Return a slice of this `BTree` containing all keys which begin with the given prefix.
         """
 
-        if not isinstance(prefix, Tuple) and not isinstance(prefix, tuple):
+        if not isinstance(prefix, (Tuple, tuple)):
             prefix = (prefix,)
 
         range = _handle_range(prefix)
@@ -88,12 +89,12 @@ class BTree(Collection):
 
 
 def _handle_range(range):
-    if range is None or isinstance(range, Ref) or isinstance(range, URI):
+    if range is None or isinstance(range, (Ref, URI)):
         return range
 
     if isinstance(range, State):
         form = form_of(range)
-        if isinstance(form, list) or isinstance(form, tuple):
+        if isinstance(form, (list, tuple)):
             range = form
         else:
             return range
