@@ -1,13 +1,14 @@
-from tinychain.collection.btree import BTree
-from tinychain.decorators import closure, delete_op, get_op, post_op
-from tinychain.error import BadRequest
-from tinychain.ref import Delete, If, Ref
-from tinychain.state import Map, Tuple, State, Stream
-from tinychain.util import form_of, to_json, uri, Context, URI
-from tinychain.value import Bool, UInt, Nil
+from ..decorators import closure, delete, get
+from ..error import BadRequest
+from ..generic import Map, Tuple
+from ..scalar.bound import Range
+from ..scalar.number import Bool, UInt
+from ..scalar.ref import If, Ref
+from ..state import State, Stream
+from ..util import form_of, to_json, uri, URI
 
-from .collection import Collection
-from .bound import Range
+from .base import Collection
+from .btree import BTree
 
 
 class Schema(object):
@@ -49,7 +50,7 @@ class Table(Collection):
         """
 
         @closure(self)
-        @get_op
+        @get
         def group(cxt, key: Tuple) -> Tuple:
             cxt.where = Tuple(columns).zip(key).cast(Map)
             cxt.slice = self.where(cxt.where)
@@ -82,7 +83,10 @@ class Table(Collection):
         If no where clause is specified, all contents of this `Table` will be deleted.
         """
 
-        delete_row = closure(self)(delete_op(lambda cxt, key: self.delete_row(key)))
+        delete_row = delete(lambda cxt, key: self.delete_row(key))
+        if uri(self).id():
+            delete_row = closure(self)(delete_row)
+
         to_delete = self.where(where) if where else self
         return to_delete.select(self.key_names()).rows().for_each(delete_row)
 
@@ -156,7 +160,7 @@ class Table(Collection):
     def update(self, values, where={}):
         """Update the specified rows of this table with the given `values`."""
 
-        update_row = closure(self)(get_op(lambda cxt, key: self.update_row(key, values)))
+        update_row = closure(self)(get(lambda cxt, key: self.update_row(key, values)))
         return self.where(where).select(self.key_names()).index().keys().for_each(update_row)
 
     def update_row(self, key, values):
@@ -189,7 +193,7 @@ def _handle_bounds(bounds):
         return {}
     elif isinstance(bounds, State):
         return _handle_bounds(form_of(bounds))
-    elif isinstance(bounds, Ref) or isinstance(bounds, URI):
+    elif isinstance(bounds, (Ref, URI)):
         return bounds
 
     return {
