@@ -1,55 +1,90 @@
 """An n-dimensional array of numbers."""
 
-from tinychain import ref
-from tinychain.reflect import is_ref
-from tinychain.state import Class, State, Stream, Tuple
-from tinychain.util import form_of, to_json, uri, URI
-from tinychain.value import Bool, I16, F32, F64, Number, UInt
+import logging
+import typing
 
-from .bound import Range
-from .collection import Collection
+from ..decorators import post
+from ..generic import Map, Tuple
+from ..interface import Equality, Interface, Order
+from ..math.interface import Numeric, Trigonometric
+from ..math.operator import Operator, Add, Div, Exp, MatMul, Mul, Pow, Sub, Sin, Cos, Asin, Acos, Sinh, Cosh, Asinh, Acosh, Tan, Tanh, Atan, Atanh
+from ..scalar.bound import handle_bounds
+from ..scalar.number import Bool, F32, F64, Number, UInt, U64
+from ..scalar import ref
+from ..state import Class, Stream
+from ..util import deanonymize, hex_id, form_of, to_json, uri, URI
+
+from .base import Collection
 
 
-class Schema(object):
-    """
-    A `Tensor` schema which comprises a shape and data type.
+class NDArray(Interface):
+    def expand_dims(self, axis=-1):
+        return ref.Get(ref.MethodSubject(self, "expand_dims"), axis)
 
-    The data type must be a subclass of `Number` and defaults to `F32`.
-    """
+    def flip(self, axis):
+        return ref.Get(ref.MethodSubject(self, "flip"), axis)
 
-    def __init__(self, shape, dtype=F32):
-        self.shape = shape
-        self.dtype = dtype
+    def reshape(self, shape):
+        return ref.Get(ref.MethodSubject(self, "reshape"), shape)
 
-    def __getitem__(self, i):
-        if i == 0:
-            return self.shape
-        elif i == 1:
-            return self.dtype
-        else:
-            raise KeyError(f"Tensor schema (shape, dtype) has no element {i}")
-
-    def __json__(self):
-        return to_json([self.shape, self.dtype])
-
-    def __repr__(self):
-        return f"tensor schema with shape {self.shape} and dtype {self.dtype}"
+    def transpose(self, permutation=None):
+        return ref.Get(ref.MethodSubject(self, "transpose"), permutation)
 
 
-class Tensor(Collection):
+class Tensor(Collection, Equality, Numeric, Order, Trigonometric, NDArray):
     """An n-dimensional array of numbers."""
 
     __uri__ = uri(Collection) + "/tensor"
+    __spec__ = (typing.Tuple[U64, ...], Number)
+
+    @classmethod
+    def trig_rtype(cls):
+        shape, dtype = cls.__spec__
+        return cls.expect(shape, dtype.trig_rtype())
+
+    @classmethod
+    def expect(cls, shape, dtype):
+        spec = (shape, dtype)
+
+        if not isinstance(shape, Tuple):
+            shape = Tuple(shape)
+
+        class _Tensor(cls):
+            __spec__ = spec
+
+            @classmethod
+            def create(cls):
+                return cls(ref.Get(cls, (shape, dtype)))
+
+            @property
+            def dtype(self):
+                return dtype
+
+            @property
+            def ndim(self):
+                if hasattr(shape, "__len__"):
+                    return len(shape)
+                else:
+                    return ref.Get(ref.MethodSubject(self, "ndim"))
+
+            @property
+            def schema(self):
+                return shape, dtype
+
+            @property
+            def shape(self):
+                return shape
+
+        return _Tensor
 
     @classmethod
     def create(cls, shape, dtype=F32):
         """Create a new, empty `Tensor`. Call this method to initialize a persistent `Tensor` in a `Chain`."""
 
-        schema = Schema(shape, dtype)
-        return cls(Create(cls, schema, schema))
+        return cls.expect(shape, dtype).create()
 
     @classmethod
-    def load(cls, shape, dtype, data):
+    def load(cls, shape, data, dtype=F32):
         """
         Load a `Tensor` from an existing data set.
 
@@ -59,17 +94,23 @@ class Tensor(Collection):
 
                 coords = [[0, 0, 1], [0, 1, 0]]
                 values = [1, 2]
-                sparse = tc.tensor.Sparse.load([2, 3, 4], tc.I32, zip(coords, values))
-                dense = tc.tensor.Dense.load([2, 3, 4], tc.i32, values)
+                sparse = tc.tensor.Sparse.load([2, 3, 4], zip(coords, values))
+                dense = tc.tensor.Dense.load([2, 3, 4], values, tc.I32)
         """
 
-        return super().load(Schema(shape, dtype), data)
+        return cls.expect(shape, dtype)(ref.Get(uri(cls) + "/load", ((shape, dtype), data)))
+
+    @classmethod
+    def zeros_like(cls, tensor):
+        """Return a `Tensor` filled with zeros, with the same shape and data type as the given `tensor`."""
+
+        return cls.expect(tensor.shape, tensor.dtype)((tensor.shape, tensor.dtype))
 
     def __getitem__(self, bounds):
         parent = self
-        bounds = _handle_bounds(bounds)
+        bounds = handle_bounds(bounds)
 
-        class Slice(self.__class__):
+        class Slice(Tensor):
             def write(self, value):
                 return parent._put("", bounds, value)
 
@@ -78,86 +119,11 @@ class Tensor(Collection):
     def __setitem__(self, bounds, value):
         raise NotImplementedError("use Tensor.write instead")
 
-    def __add__(self, other):
-        return self.add(other)
-
-    def __abs__(self):
-        return self.abs()
-
-    def __radd__(self, other):
-        return self.add(other)
-
-    def __eq__(self, other):
-        return self.eq(other)
-
-    def __gt__(self, other):
-        return self.gt(other)
-
-    def __ge__(self, other):
-        return self.gte(other)
-
-    def __lt__(self, other):
-        return self.lt(other)
-
-    def __le__(self, other):
-        return self.lte(other)
-
-    def __mul__(self, other):
-        return self.mul(other)
-
-    def __rmul__(self, other):
-        return self.mul(other)
-
-    def __ne__(self, other):
-        return self.ne(other)
-
-    def __neg__(self):
-        return self * -1
-
-    def __pow__(self, other):
-        return self.pow(other)
-
-    def __sub__(self, other):
-        return self.sub(other)
-
-    def __rsub__(self, other):
-        return -(self - other)
-
-    def __truediv__(self, other):
-        return self.div(other)
-
-    def __rtruediv__(self, other):
-        return other * (self.pow(-1))
-
-    def __ref__(self, name):
-        form = form_of(self)
-        if hasattr(form, "schema"):
-            ref_form = TypeRef(URI(name), form.schema)
-            return self.__class__(ref_form)
-        elif hasattr(form, "__ref__"):
-            return self.__class__(get_ref(form, name))
-        else:
-            return self.__class__(URI(name))
-
-    def abs(self):
-        """Return the element-wise absolute value of this `Tensor`."""
-
-        return self._get("abs", rtype=self.__class__)
-
-    def acos(self):
-        """Return the element-wise arccosine of this `Tensor`."""
-
-        return self._get("acos", rtype=self.__class__)
-
-    def acosh(self):
-        """Return the element-wise hyperbolic arccosine of this `Tensor`."""
-
-        return self._get("acosh", rtype=self.__class__)
+    def __matmul__(self, other):
+        return Tensor(MatMul(self, other))
 
     def add(self, other):
-        """Return the element-wise sum of this `Tensor` and another `Tensor` or `Number`."""
-
-        return self._post("add", {"r": other}, Tensor)
+        return Tensor(Add(self, other))
 
     def all(self):
         """Return `True` if all elements in this `Tensor` are nonzero."""
@@ -177,50 +143,18 @@ class Tensor(Collection):
 
         return self._get("argmax", axis, self.__class__)
 
-    def asin(self):
-        """Return the element-wise arcsine of this `Tensor`."""
-
-        return self._get("asin", rtype=self.__class__)
-
-    def asinh(self):
-        """Return the element-wise hyperbolic arcsine of this `Tensor`."""
-
-        return self._get("asinh", rtype=self.__class__)
-
-    def atan(self):
-        """Return the element-wise arctangent of this `Tensor`."""
-
-        return self._get("atan", rtype=self.__class__)
-
-    def atanh(self):
-        """Return the element-wise hyperbolic arctangent of this `Tensor`."""
-
-        return self._get("atanh", rtype=self.__class__)
-
     def cast(self, number_type):
         """Cast the data type of `Tensor` into the given `number_type`."""
 
-        return self._get("cast", number_type, self.__class__)
+        return self._get("cast", number_type, self.__class__.expect(self.shape, number_type))
 
     def copy(self):
         """Return a copy of this `Tensor`"""
 
         return self.__class__(ref.Post(uri(Tensor) + "/copy_from", {"tensor": self}))
 
-    def cos(self):
-        """Return the element-wise cosine of this `Tensor`."""
-
-        return self._get("cos", rtype=self.__class__)
-
-    def cosh(self):
-        """Return the element-wise hyperbolic cosine of this `Tensor`."""
-
-        return self._get("cosh", rtype=self.__class__)
-
     def div(self, other):
-        """Divide this `Tensor` by another `Tensor` or `Number`, broadcasting if necessary."""
-
-        return self._post("div", {"r": other}, Tensor)
+        return Tensor(Div(self, other))
 
     @property
     def dtype(self):
@@ -228,49 +162,81 @@ class Tensor(Collection):
 
         return self._get("dtype", rtype=Class)
 
+    def exp(self):
+        return Tensor(Exp(self))
+
+    def sin(self):
+        return Tensor(Sin(self))
+
+    def cos(self):
+        return Tensor(Cos(self))
+
+    def asin(self):
+        return Tensor(Asin(self))
+
+    def acos(self):
+        return Tensor(Acos(self))
+
+    def sinh(self):
+        return Tensor(Sinh(self))
+
+    def cosh(self):
+        return Tensor(Cosh(self))
+
+    def asinh(self):
+        return Tensor(Asinh(self))
+
+    def acosh(self):
+        return Tensor(Acosh(self))
+    
+    def tan(self):
+        return Tensor(Tan(self))
+
+    def tanh(self):
+        return Tensor(Tanh(self))
+
+    def atan(self):
+        return Tensor(Atan(self))
+
+    def atanh(self):
+        return Tensor(Atanh(self))
+
     def flip(self, axis):
         """Flip the elements in this `Tensor` along the specified `axis`."""
 
-        return self._get("flip", axis, Tensor)
+        return self.__class__(Flip(self, axis))
 
     def eq(self, other):
         """Return a boolean `Tensor` with element-wise equality values."""
 
         return self._post("eq", {"r": other}, Tensor)
 
-    def exp(self):
-        """Raise `e` to the power of this `Tensor`."""
-
-        return self._get("exp", rtype=self.__class__)
-
     def expand_dims(self, axis=None):
         """Return a view of this `Tensor` with an extra dimension of size 1 at the given axis."""
 
-        return self._get("expand_dims", axis, self.__class__)
+        rtype = Tensor
+        if isinstance(form_of(self.shape), (list, tuple)):
+            if isinstance(form_of(axis), int):
+                shape = list(form_of(self.shape))
+                shape.insert(form_of(axis), 1)
+                rtype = Tensor.expect(shape, self.dtype)
+
+        return rtype(form=Expand(self, axis))
 
     def gt(self, other):
         """Return a boolean `Tensor` with element-wise greater-than values."""
 
-        return self._post("gt", {"r": other}, self.__class__)
+        return self._post("gt", {"r": other}, Tensor)
 
     def gte(self, other):
         """Return a boolean `Tensor` with element-wise greater-or-equal values."""
 
         return self._post("gte", {"r": other}, Tensor)
 
-    def log(self, base=None):
-        """
-        Return the logarithm of this `Tensor`.
-
-        If no `base` is specified, this will return the natural logarithm (base e).
-        """
-
-        return self._get("log", base, self.__class__)
-
     def lt(self, other):
         """Return a boolean `Tensor` with element-wise less-than values."""
 
-        return self._post("lt", {"r": other}, self.__class__)
+        return self._post("lt", {"r": other}, Tensor)
 
     def lte(self, other):
         """Return a boolean `Tensor` with element-wise less-or-equal values."""
@@ -280,7 +246,7 @@ class Tensor(Collection):
     def logical_and(self, other):
         """Return a boolean `Tensor` with element-wise logical and values."""
 
-        return self._post("and", {"r": other}, self.__class__)
+        return self._post("and", {"r": other}, Tensor)
 
     def logical_not(self):
         """Return a boolean `Tensor` with element-wise logical not values."""
@@ -315,57 +281,32 @@ class Tensor(Collection):
             return self.sum(axis) / self.shape[axis]
 
     def mul(self, other):
-        """Multiply this `Tensor` by another `Tensor` or `Number`, broadcasting if necessary."""
-
-        return self._post("mul", {"r": other}, self.__class__)
+        return Tensor(Mul(self, other))
 
     def ne(self, other):
         """Return a boolean `Tensor` with element-wise not-equal values."""
 
-        return self._post("ne", {"r": other}, self.__class__)
+        return self._post("ne", {"r": other}, Tensor)
 
     def pow(self, other):
-        """Raise this `Tensor` to the given power."""
-
-        return self._post("pow", {"r": other}, self.__class__)
+        return Tensor(Pow(self, other))
 
     def product(self, axis=None):
         """Calculate the product of this `Tensor` along the given `axis`, or the total product if no axis is given."""
 
-        rtype = Number if axis is None else self.__class__
+        rtype = Number if axis is None else Tensor
         return self._get("product", axis, rtype)
 
     def reshape(self, shape):
         """Return a view of this `Tensor` with the given `shape`."""
 
-        return self._get("reshape", shape, self.__class__)
-
-    def round(self):
-        """Round this `Tensor` to the nearest integer, element-wise."""
-
-        return self._get("round", rtype=self.__class__)
+        return Tensor.expect(shape, self.dtype)(form=Reshape(self, shape))
 
     @property
     def shape(self):
         """Return the shape of this `Tensor`."""
 
-        form = form_of(self)
-        if hasattr(form, "schema"):
-            shape = form.schema[0]
-            shape = shape if isinstance(shape, Tuple) else Tuple(shape)
-            return shape
-        else:
-            return self._get("shape", rtype=Tuple)
-
-    def sin(self):
-        """Return the element-wise sine of this `Tensor`."""
-
-        return self._get("sin", rtype=self.__class__)
-
-    def sinh(self):
-        """Return the element-wise hyperbolic sine of this `Tensor`."""
-
-        return self._get("sinh", rtype=self.__class__)
+        return self._get("shape", rtype=Tuple.expect(typing.Tuple[U64, ...]))
 
     @property
     def size(self):
@@ -385,29 +326,30 @@ class Tensor(Collection):
         `self.shape[axis]` then a `BadRequest` error will be raised.
         """
 
-        return self._get("split", (num_or_size_splits, axis), Tuple)
+        return self._get("split", (num_or_size_splits, axis), typing.Tuple[Tensor, ...])
+
+    def std(self, axis=None):
+        """
+        Return the standard deviation of this `Tensor` along the given `axis`,
+        or the standard deviation of the entire `Tensor` if no axis is given.
+        """
+
+        if axis is None:
+            average = self.mean()
+            size = self.size
+            return (((self - average)**2).sum() / size)**0.5
+        else:
+            raise NotImplementedError("Tensor.std with axis")
 
     def sub(self, other):
-        """Subtract another `Tensor` or `Number` from this one, broadcasting if necessary."""
-
-        return self._post("sub", {"r": other}, Tensor)
+        return Tensor(Sub(self, other))
 
     def sum(self, axis=None):
         """Calculate the sum of this `Tensor` along the given `axis`, or the total sum if no axis is given."""
 
-        rtype = Number if axis is None else self.__class__
+        rtype = Number if axis is None else Tensor
         return self._get("sum", axis, rtype)
 
-    def tan(self):
-        """Return the element-wise tangent of this `Tensor`."""
-
-        return self._get("tan", rtype=self.__class__)
-
-    def tanh(self):
-        """Return the element-wise hyperbolic tangent of this `Tensor`."""
-
-        return self._get("tanh", rtype=self.__class__)
-
     def transpose(self, permutation=None):
         """
         Return a view of this `Tensor` with its axes transposed according to the given permutation.
@@ -415,7 +357,21 @@ class Tensor(Collection):
         If no permutation is given, the axes will be inverted (e.g. `(0, 1, 2)` inverts to `(2, 1, 0)`).
         """
 
-        return self._get("transpose", permutation, self.__class__)
+        dtype = self.dtype
+        shape = None
+
+        if hasattr(self.shape, "__len__"):
+            if permutation is None:
+                shape = reversed(self.shape)
+            elif hasattr(permutation, "__iter__"):
+                shape = [self.shape[x] for x in permutation]
+
+        if shape is None:
+            rtype = Tensor
+        else:
+            rtype = Tensor.expect(shape, dtype)
+
+        return rtype(form=Transpose(self, permutation))
 
     def write(self, value):
         """Overwrite this `Tensor` with the given `Tensor` or `Number`, broadcasting if needed."""
@@ -442,8 +398,7 @@ class Dense(Tensor):
         """
 
         dtype = type(start) if isinstance(start, Number) else Number
-        schema = Schema(shape, dtype)
-        return cls(Create(uri(cls) + "/range", (shape, start, stop), schema))
+        return cls.expect(shape, dtype)(ref.Get(uri(cls) + "/range", (shape, start, stop)))
 
     @classmethod
     def concatenate(cls, tensors, axis=None):
@@ -460,45 +415,7 @@ class Dense(Tensor):
         """Return a `Dense` tensor filled with the given `value`."""
 
         dtype = type(value) if isinstance(value, Number) else Number
-        schema = Schema(shape, dtype)
-        return cls(Create(uri(cls) + "/constant", (shape, value), schema))
-
-    @classmethod
-    def load(cls, shape, dtype, data):
-        """
-        Load a `Dense` tensor from an existing data set.
-
-        Example: `tc.tensor.Dense.load([2, 2], tc.i32, [1, 2, 3, 4])`
-        """
-
-        if is_ref(shape) or is_ref(dtype):
-            raise ValueError(f"cannot load schema ({shape}, {dtype}) (consider calling `copy_from` instead)")
-
-        if is_ref(data):
-            raise ValueError(f"cannot load data {data} (consider calling `copy_from` instead)")
-
-        schema = Schema(shape, dtype)
-
-        class Load(cls):
-            def __init__(self, put_op):
-                cls.__init__(self, put_op)
-
-            @property
-            def dtype(self):
-                return schema[1]
-
-            @property
-            def schema(self):
-                return schema
-
-            @property
-            def shape(self):
-                return schema[0]
-
-            def __ref__(self, name):
-                return cls(URI(name))
-
-        return Load(ref.Put(cls, schema, data))
+        return cls.expect(shape, dtype)(ref.Get(uri(cls) + "/constant", (shape, value)))
 
     @classmethod
     def ones(cls, shape, dtype=F32):
@@ -508,7 +425,13 @@ class Dense(Tensor):
         If `dtype` is not specified, the data type will be :class:`F32`.
         """
 
-        return cls.constant(shape, Number(1).cast(dtype))
+        return cls.expect(shape, dtype).constant(shape, Number(1).cast(dtype))
+
+    @classmethod
+    def ones_like(cls, tensor):
+        """Return a `Dense` tensor filled with ones, with the same shape and data type as the given `tensor`."""
+
+        return cls.expect(tensor.shape, tensor.dtype).constant(tensor.shape, Number(1).cast(tensor.dtype))
 
     @classmethod
     def zeros(cls, shape, dtype=F32):
@@ -518,39 +441,85 @@ class Dense(Tensor):
         If `dtype` is not specified, the data type will be :class:`F32`.
         """
 
-        return cls.constant(shape, Number(0).cast(dtype))
+        return cls.expect(shape, dtype).constant(shape, Number(0).cast(dtype))
 
     @classmethod
     def random_normal(cls, shape, mean=0.0, std=1.0):
         """Return a `Dense` tensor filled with a random normal distribution of `F64` s."""
 
-        schema = Schema(shape, F64)
         args = {"shape": shape, "mean": mean, "std": std}
-        return cls(Distribution(uri(cls) + "/random/normal", args, schema))
+        return cls.expect(shape, F64)(ref.Post(uri(cls) + "/random/normal", args))
 
     @classmethod
-    def random_uniform(cls, shape):
+    def random_uniform(cls, shape, minval=0, maxval=1):
         """Return a `Dense` tensor filled with a uniform random distribution of `F64` s."""
 
-        schema = Schema(shape, F64)
-        return cls(Create(uri(cls) + "/random/uniform", shape, schema))
+        if minval == maxval:
+            return cls.constant(shape, minval)
+
+        assert maxval > minval
+
+        random = cls(ref.Get(uri(cls) + "/random/uniform", shape))
+        if (minval, maxval) == (0, 1):
+            return random
+        else:
+            range = maxval - minval
+            return (random * range) + minval
+
+    @classmethod
+    def truncated_normal(cls, shape, mean=0.0, std=1.0, minval=None, maxval=None):
+        """
+        Return a `Dense` tensor filled with a random normal distribution of `F64` s.
+
+        Any value `x` outside the range `minval <= x <= maxval` will be replaced by a value drawn from a new
+        random normal distribution.
+
+        `minval` and `maxval` default to two standard deviations.
+        """
+
+        if not std:
+            return cls.constant(shape, mean)
+
+        minval = std * -2 if minval is None else minval
+        maxval = std * 2 if maxval is None else maxval
+
+        @post
+        def cond(cxt, tensor: Dense) -> Bool:
+            cxt.too_small = (tensor < minval).any()
+            cxt.too_big = (tensor > maxval).any()
+            return cxt.too_small.logical_or(cxt.too_big)
+
+        @post
+        def step(cxt, tensor: Dense) -> Map:
+            cxt.new_dist = Dense.random_normal(shape, mean, std)
+            cxt.new_tensor = where((tensor >= minval).logical_and(tensor <= maxval), tensor, cxt.new_dist)
+            return Map(tensor=cxt.new_tensor.copy())
+
+        truncated = Map(ref.While(cond, step, Map(tensor=Dense.random_normal(shape, mean, std))))["tensor"]
+        return cls(truncated)
+
+    def add(self, other):
+        return Dense(Add(self, other))
 
     def argsort(self):
         """Return the coordinates needed to sort this `Tensor`."""
 
         return self._get("argsort", rtype=self.__class__)
 
-    def elements(self, bounds):
-        """Return a :class:`Stream` of the :class:`Number` elements of this `Dense` tensor."""
-
-        bounds = _handle_bounds(bounds)
-        return self._get("elements", bounds, Stream)
-
     def as_sparse(self):
         """Return a :class:`Sparse` view of this `Dense` tensor."""
 
         return self._get("sparse", rtype=Sparse)
 
+    def elements(self, bounds=None):
+        """Return a :class:`Stream` of the :class:`Number` elements of this `Dense` tensor."""
+
+        bounds = handle_bounds(bounds)
+        return self._get("elements", bounds, Stream)
+
+    def sub(self, other):
+        return Dense(Sub(self, other))
+
 
 class Sparse(Tensor):
     """
@@ -574,19 +543,29 @@ class Sparse(Tensor):
         If `dtype` is not specified, the data type will be :class:`F32`.
         """
 
-        return cls(Schema(shape, dtype))
-
-    def elements(self, bounds=None):
-        """Return a :class:`Stream` of this tensor's (:class:`Tuple`, :class:`Number`) coordinate-value elements."""
-
-        bounds = _handle_bounds(bounds)
-        return self._get("elements", bounds, Stream)
+        return cls.expect(shape, dtype)(ref.Get(cls, (shape, dtype)))
 
     def as_dense(self):
         """Return a :class:`Dense` view of this `Sparse` tensor."""
 
         return self._get("dense", rtype=Dense)
 
+    def div(self, other):
+        return Sparse(Div(self, other))
+
+    def elements(self, bounds=None):
+        """
+        Return a :class:`Stream` of this tensor's `(coord, number)` coordinate-value elements.
+
+        `coord` is a :class:`Tuple` of :class:`U64` coordinates, and `number` is the element at `coord`.
+        """
+
+        bounds = handle_bounds(bounds)
+        return self._get("elements", bounds, Stream)
+
+    def mul(self, other):
+        return Sparse(Mul(self, other))
+
 
 def einsum(format, tensors):
     """
@@ -612,43 +591,63 @@ def tile(tensor, multiples):
     return rtype(ref.Post(uri(Tensor) + "/tile", {"tensor": tensor, "multiples": multiples}))
 
 
-def _handle_bounds(bounds):
-    if bounds is None or isinstance(bounds, ref.Ref) or isinstance(bounds, URI):
-        return bounds
+def where(cond, x, y):
+    """
+    Return a view of `x` and `y` depending on whether the corresponding element of `cond` is `True`.
 
-    if isinstance(bounds, State):
-        form = form_of(bounds)
-        if isinstance(form, tuple) or isinstance(form, list):
-            bounds = form
+    `cond`, `x`, and `y` must support broadcasting to the same shape.
+    """
+
+    return (cond.cast(Bool) * x) + (cond.logical_not() * y)
+
+
+class Transform(Operator):
+    def backward(self, variable):
+        raise RuntimeError(f"{self.__class__.__name__} is a tensor transform and has no derivative")
+
+    def gradients(self, loss):
+        if not isinstance(form_of(self.subject), Operator):
+            logging.info(f"{self.subject} is assumed to be constant and has no gradient")
+            return {}
+
+        return form_of(self.subject).gradients(loss)
+
+
+class Expand(Transform):
+    def forward(self):
+        return NDArray.expand_dims(self.subject, self.args)
+
+    def gradients(self, loss):
+        return Transform.gradients(self, loss.reshape(self.subject.shape))
+
+
+class Flip(Transform):
+    def forward(self):
+        return NDArray.flip(self.subject, self.args)
+
+    def gradients(self, loss):
+        return Transform.gradients(self, loss.flip(self.args))
+
+
+class Transpose(Transform):
+    def __init__(self, subject, permutation=None):
+        Transform.__init__(self, subject, permutation)
+
+        if permutation is None:
+            self.inverse_permutation = None
         else:
-            return bounds
+            self.inverse_permutation = tuple(i for _x, i in sorted((x, i) for i, x in enumerate(permutation)))
 
-    if hasattr(bounds, "__iter__"):
-        bounds = tuple(bounds)
-    else:
-        bounds = (bounds,)
+    def forward(self):
+        return NDArray.transpose(self.subject, self.args)
 
-    return [
-        Range.from_slice(x) if isinstance(x, slice)
-        else x for x in bounds]
+    def gradients(self, loss):
+        return Transform.gradients(self, loss.transpose(self.inverse_permutation))
 
 
-class Create(ref.Get):
-    def __init__(self, subject, args, schema):
-        ref.Get.__init__(self, subject, args)
-        self.schema = schema
+class Reshape(Transform):
+    def forward(self):
+        return NDArray.reshape(self.subject, self.args)
 
-
-class Distribution(ref.Get):
-    def __init__(self, subject, args, schema):
-        ref.Post.__init__(self, subject, args)
-        self.schema = schema
-
-
-class TypeRef(ref.Ref):
-    def __init__(self, name, schema):
-        self.__uri__ = name
-        self.schema = schema
-
-    def __json__(self):
-        return to_json(self.__uri__)
+    def gradients(self, loss):
+        return Transform.gradients(self, loss.reshape(self.subject.shape))
