@@ -1,12 +1,13 @@
 """Utilities for communicating with a TinyChain host."""
+
 import abc
 import json
 import requests
 import urllib.parse
 
-from tinychain.error import *
-from tinychain.util import to_json, uri, URI
-from tinychain.value import Nil
+from .error import *
+from .scalar.value import Nil
+from .util import to_json, uri, URI
 
 
 DEFAULT_PORT = 8702
@@ -64,6 +65,11 @@ class Host(object):
     def link(self, path):
         """Return a link to the given path at this host."""
 
+        if hasattr(path, "__uri__"):
+            path = uri(path)
+        else:
+            path = URI(path)
+
         return uri(self) + path
 
     def get(self, path, key=None, auth=None):
