@@ -1,11 +1,11 @@
 import inspect
 
-from tinychain import op, ref
-from tinychain.state import State
-from tinychain.util import form_of, to_json, uri, Context, URI
-from tinychain.value import Nil, Value
+from ..scalar import Value
+from ..scalar import op, ref
+from ..state import State
+from ..util import form_of, to_json, uri, Context, URI
 
-from . import _get_rtype, is_none, resolve_class
+from . import _get_rtype, is_none, parse_args, resolve_class
 
 
 EMPTY = inspect.Parameter.empty
@@ -22,6 +22,12 @@ class Method(object):
         self.form = form
         self.name = name
 
+    def __eq__(self, other):
+        if isinstance(other, Method):
+            return self.form == other.form
+
+        return False
+
     def __json__(self):
         return {str(uri(self)): to_json(form_of(self))}
 
@@ -40,12 +46,12 @@ class Get(Method):
 
     def __init__(self, header, form, name):
         Method.__init__(self, header, form, name)
+        self.rtype = _get_rtype(self.form, State)
 
     def __call__(self, key=None):
-        rtype = _get_rtype(self.form, State)
-        return rtype(ref.Get(self.subject(), key))
+        return self.rtype(ref.Get(self.subject(), key))
 
-    def __dbg__(self):
+    def __args__(self):
         _, cxt = form_of(self)
         return [self.subject(), cxt]
 
@@ -60,11 +66,15 @@ class Get(Method):
             key_name = list(sig.parameters.keys())[-1]
             param = sig.parameters[key_name]
             dtype = resolve_class(self.form, param.annotation, Value)
-            args.append(dtype(URI(key_name)))
+            args.append(dtype(form=URI(key_name)))
         else:
             raise ValueError(f"{self.dtype()} takes 0-3 parameters: (self, cxt, key)")
 
+        if key_name in cxt:
+            raise RuntimeError(f"namespace collision: {key_name} in {self.form}")
+
         cxt._return = self.form(*args)  # populate the Context
+
         return key_name, cxt
 
 
@@ -75,14 +85,14 @@ class Put(Method):
         rtype = _get_rtype(form, None)
 
         if not is_none(rtype):
-            raise ValueError(f"Put method must return None, not f{rtype}")
+            raise ValueError(f"PUT method must return None, not f{rtype}")
 
         Method.__init__(self, header, form, name)
 
     def __call__(self, key, value):
         return ref.Put(self.subject(), key, value)
 
-    def __dbg__(self):
+    def __args__(self):
         _, _, cxt = form_of(self)
         return [self.subject(), cxt]
 
@@ -101,10 +111,10 @@ class Put(Method):
             param = sig.parameters[name]
             if name == key_name:
                 dtype = resolve_class(self.form, param.annotation, Value)
-                args.append(dtype(URI(key_name)))
+                args.append(dtype(form=URI(key_name)))
             elif name == value_name:
                 dtype = resolve_class(self.form, param.annotation, State)
-                args.append(dtype(URI(value_name)))
+                args.append(dtype(form=URI(value_name)))
             else:
                 raise ValueError(f"{self.dtype()} with three parameters requires 'key' or 'value', not '{name}'")
         elif len(sig.parameters) - len(args) == 2:
@@ -112,31 +122,48 @@ class Put(Method):
 
             param = sig.parameters[key_name]
             dtype = resolve_class(self.form, param.annotation, Value)
-            args.append(dtype(URI(key_name)))
+            args.append(dtype(form=URI(key_name)))
 
             param = sig.parameters[value_name]
             dtype = resolve_class(self.form, param.annotation, State)
-            args.append(dtype(URI(value_name)))
+            args.append(dtype(form=URI(value_name)))
         else:
             raise ValueError(f"{self.dtype()} requires 0-4 parameters: (self, cxt, key, value)")
 
         cxt._return = self.form(*args)
+
+        for name in [key_name, value_name]:
+            if name in cxt:
+                raise RuntimeError(f"namespace collision: {name} in {self.form}")
+
         return key_name, value_name, cxt
 
 
 class Post(Method):
     __uri__ = uri(op.Post)
 
-    def __call__(self, params=None, **kwargs):
-        if params is None:
-            params = kwargs
-        elif kwargs:
-            raise ValueError("Post method takes a single argument called `params`, or kwargs, not both")
+    def __init__(self, header, form, name):
+        Method.__init__(self, header, form, name)
+        self.rtype = _get_rtype(self.form, State)
 
-        rtype = _get_rtype(self.form, State)
+    def __call__(self, *args, **kwargs):
+        sig = list(inspect.signature(self.form).parameters.items())
+        rtype = self.rtype
+
+        if not sig:
+            raise TypeError(f"POST method signature for {self} is missing the 'self' parameter")
+        elif sig[0][0] != "self":
+            raise TypeError(f"POST method signature must begin with 'self', not '{sig[0][0]}'")
+
+        if len(sig) > 1 and sig[1][0] in ["cxt", "txn"]:
+            sig = sig[2:]
+        else:
+            sig = sig[1:]
+
+        params = parse_args(sig, *args, **kwargs)
         return rtype(ref.Post(self.subject(), params))
 
-    def __dbg__(self):
+    def __args__(self):
         _, cxt = form_of(self)
         return [self.subject(), cxt]
 
@@ -148,9 +175,14 @@ class Post(Method):
         for name in list(sig.parameters.keys())[len(args):]:
             param = sig.parameters[name]
             dtype = resolve_class(self.form, param.annotation, State)
-            kwargs[name] = dtype(URI(name))
+            kwargs[name] = dtype(form=URI(name))
 
         cxt._return = self.form(*args, **kwargs)
+
+        for name in kwargs.keys():
+            if name in cxt:
+                raise RuntimeError(f"namespace collision: {name} in {self.form}")
+
         return cxt
 
 
@@ -161,14 +193,14 @@ class Delete(Method):
         rtype = _get_rtype(form, None)
 
         if not is_none(rtype):
-            raise ValueError(f"Delete method must return None, not f{rtype}")
+            raise ValueError(f"DELETE method must return None, not f{rtype}")
 
         Method.__init__(self, header, form, name)
 
     def __call__(self, key=None):
         return ref.Delete(self.subject(), key)
 
-    def __dbg__(self):
+    def __args__(self):
         _, cxt = form_of(self)
         return [self.subject(), cxt]
 
@@ -176,6 +208,11 @@ class Delete(Method):
         return Get.__form__(self)
 
 
+class Operator(Post):
+    def __call__(self, *args, **kwargs):
+        return self.form(self.header, *args, **kwargs)
+
+
 def _check_context_param(parameter):
     _name, param = parameter
     if param.annotation == EMPTY or param.annotation == Context:
